/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __BASE_STRING_H
#define __BASE_STRING_H

#include "object.h"
#include "math.consts.h"

# if !defined(__FORWINDOWS__)
#	include <ctype.h>
# endif

namespace magnum { namespace types {
using magnum::object::Object;
using magnum::math::min;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class String : public Object {
DeclareClass(String, Object);
protected:
	char*				m_Buffer;
	unsigned int		m_BufferSize;
	unsigned int		m_Length;

public:
	inline String();
	inline String(const String& );
	inline String(const char* );
	inline ~String();

	inline String		getName() const;
	inline String		toString() const;

	inline unsigned int	getLength() const;
	inline const char*	getString() const;

	inline int			indexOf(const String& , unsigned int from = 0) const;
	inline int			indexOf(const char* , unsigned int from = 0) const;
	inline int			indexOf(char , unsigned int from = 0) const;
	inline int			lastIndexOf(const String& , unsigned int from = 0) const;
	int					lastIndexOf(const char* , unsigned int from = 0) const;
	int					lastIndexOf(char , unsigned int from = 0) const;
	inline bool			startsWith(const String& , unsigned int from = 0) const;
	inline bool			startsWith(const char* , unsigned int from = 0) const;
	inline String		substring(unsigned int start) const;
	inline String		substring(unsigned int start, unsigned int end) const;
	inline String		toLowerCase() const;
	inline String		toUpperCase() const;

	inline String		trim();

	inline bool 		operator==(const String& str) const;
	inline bool 		operator==(const char* ) const;
	inline bool 		operator!=(const String& str) const;
	inline bool 		operator!=(const char* ) const;
	inline bool 		operator>(const String& str) const;
	inline bool 		operator>(const char* ) const;
	inline bool 		operator<(const String& str) const;
	inline bool 		operator<(const char* ) const;
	inline bool 		operator>=(const String& str) const;
	inline bool 		operator>=(const char* ) const;
	inline bool 		operator<=(const String& str) const;
	inline bool 		operator<=(const char* ) const;

	const String& 		operator+=(const String& str);
	const String& 		operator+=(const char* str);
	const String& 		operator+=(const Object& str);

	const String& 		operator=(const String& str);
	const String& 		operator=(const char* str);
	const String& 		operator=(const Object& str);

	inline				operator const char*() const;

	friend String 		operator+(const String& str1, const String& str2);
	friend String 		operator+(const String& str1, const char* str2);
	friend String 		operator+(const char* str1, const String& str2);
	friend String 		operator+(const String& str1, const Object& str2);
	
private:
	void				allocBuffer();
};



inline String operator+(const String& str1, const String& str2);
inline String operator+(const String& str1, const char* str2);
inline String operator+(const char* str1, const String& str2);
inline String operator+(const String& str1, const Object& str2);

}; };
#endif
