/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __GEOMETRY_ALIGNEDBOX_INL
#define __GEOMETRY_ALIGNEDBOX_INL

namespace magnum { namespace geometry {
using magnum::math::max;
using magnum::math::min;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline AlignedBox<T,dim>::AlignedBox() {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline AlignedBox<T,dim>::AlignedBox(const Vector<T,dim>& min, const Vector<T,dim>& max) {
	m_Minimum = min;
	m_Maximum = max;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline void AlignedBox<T,dim>::setMinimum(const Vector<T,dim>& m) {
	m_Minimum = m;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline const Vector<T,dim>&	AlignedBox<T,dim>::getMinimum() const {
	return m_Minimum;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline void AlignedBox<T,dim>::setMaximum(const Vector<T,dim>& m) {
	m_Maximum = m;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline const Vector<T,dim>& AlignedBox<T,dim>::getMaximum() const {
	return m_Maximum;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline void AlignedBox<T,dim>::setOrigin(const Vector<T,dim>& o) {
	Vector<T,dim> v = o - (m_Minimum + m_Maximum)/T(2);
	m_Minimum += o;
	m_Maximum += o;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Vector<T,dim> AlignedBox<T,dim>::getOrigin() const {
	return (m_Minimum + m_Maximum)/T(2);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline void AlignedBox<T,dim>::empty() {
	m_Maximum = m_Minimum = getOrigin();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline bool AlignedBox<T,dim>::isEmpty() const {
	return (m_Minimum == m_Maxium);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline bool AlignedBox<T,dim>::isInside(const Vector<T,dim>& v) const {
	for (unsigned int i = 0; i < dim; i++) {
		if ((v[i] < m_Minimum[i]) || (v[i] > m_Maximum[i]))
			return false;
	}
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline T AlignedBox<T,dim>::getDistance(const Vector<T,dim>& ) const {
	ASSERT(false);
	return 0;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
void AlignedBox<T,dim>::merge(const AlignedBox<T,dim>& box) {
	for (unsigned int i = 0; i < dim; i++) {
		m_Minimum[i] = min(m_Minimum[i], box.m_Minimum[i]);
		m_Maximum[i] = max(m_Maximum[i], box.m_Maximum[i]);
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline void AlignedBox<T,dim>::bound(const Vector<T,dim>* v,unsigned int n) {
	bound(v, sizeof(Vector<T,dim>), n);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
void AlignedBox<T,dim>::bound(const Vector<T,dim>* v,unsigned int stride,unsigned int n) {
	m_Minimum = v[0];
	m_Maximum = v[0];
	
	for(unsigned int i = 0; i < n; i++) {
		for(unsigned int j = 0; j < dim; j++) {
			if (m_Minimum[j] > (*v)[j])
				m_Minimum[j] = (*v)[j];
			else if (m_Maximum[j] < (*v)[j])
				m_Maximum[j] = (*v)[j];
		}
		v = (const Vector<T,dim>*)(const void*)(((const byte*)v) + stride);
	}
}


}; };

#endif
