/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __GEOMETRY_BOX_INL
#define __GEOMETRY_BOX_INL

namespace magnum { namespace geometry {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim>
inline void Box<T,dim>::setOrigin(const Vector<T,dim>& o) {
	m_Origin = o;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim>
inline const Vector<T,dim>& Box<T,dim>::getOrigin() const {
	return m_Origin;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim>
inline void	Box<T,dim>::setAxes(const Vector<T,dim>* e) {
	memcpy(m_Axes, e, sizeof(m_Axes));
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim>
inline const Vector<T,dim>* Box<T,dim>::getAxes() const {
	return m_Axes;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim>
inline void	Box<T,dim>::setAxis(unsigned int i, const Vector<T,dim>& a) {
	ASSERT(i < dim);
	m_Axes[i] = a;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim>
inline const Vector<T,dim>& Box<T,dim>::getAxis(unsigned int i) const {
	ASSERT(i < dim);
	return m_Axes[i];
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim>
inline bool Box<T,dim>::isEmpty() const {
	for(unsigned int i = 0; i < dim; i++) {
		if (m_Axes[i] == Vector<T,dim>::Null)
			return true;
	}
	return false;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim>
inline void Box<T,dim>::empty() {
	for(unsigned int i = 0; i < dim; i++)
		m_Axes[i] = Vector<T,dim>::Null;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim>
inline bool Box<T,dim>::isInside(const Vector<T,dim>& v) const {
	ASSERT(false);
	return false;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim>
inline T Box<T,dim>::getDistance(const Vector<T,dim>& ) const {
	ASSERT(false);
	return false;
}


}; };

#endif
